from typing import Any, Dict, List

import requests
import yaml
from schemax_openapi import SchemaData, collect_schema_data


class SpecMethod:
    def __init__(self, method: str, route: str, query_params: List[str], response_codes: List[str]):
        self.method = method
        self.route = route
        self.query_params = query_params
        self.response_codes = response_codes

    @staticmethod
    def create(data: Any) -> "SpecMethod":
        if isinstance(data, SchemaData):
            return SpecMethod(
                method=data.http_method,
                route=data.path,
                query_params=data.queries,
                response_codes=[str(data.status)]
            )
        else:
            raise ValueError("Unsupported data format")


class Parser:
    @classmethod
    def parse(cls, spec_path: str) -> Dict[str, SpecMethod]:
        if spec_path.startswith("http://") or spec_path.startswith("https://"):
            return cls.parse_from_url(spec_path)
        else:
            return cls.parse_from_file(spec_path)

    @staticmethod
    def parse_from_url(url: str) -> Dict[str, SpecMethod]:
        response = requests.get(url)
        content = yaml.safe_load(response.text)

        result = dict()
        for data in collect_schema_data(content):
            result[data.interface_method] = SpecMethod.create(data)
        return result

    @staticmethod
    def parse_from_file(file_path: str) -> Dict[str, SpecMethod]:
        with open(file_path) as f:
            content = yaml.safe_load(f)

        result = dict()
        for data in collect_schema_data(content):
            result[data.interface_method] = SpecMethod.create(data)

        return result



from typing import Any, Dict, List, Optional

from .parser import SpecMethod


class DiffMethod:
    def __init__(self, method_data: SpecMethod, details: Optional[List[str]] = None):
        self.http_method = method_data.method
        self.http_path = method_data.route
        self.details = details or []


class Diff:
    def __init__(self) -> None:
        self.all: int = 0
        self.full: int = 0
        self.partial: int = 0
        self.empty: int = 0
        self.methods_full: List[DiffMethod] = []
        self.methods_partial: List[DiffMethod] = []
        self.methods_empty: List[DiffMethod] = []
        self.full_percent: float = 0
        self.partial_percent: float = 0
        self.empty_percent: float = 0
        self.stat_full_percent: float = 0
        self.stat_partial_percent: float = 0
        self.stat_empty_percent: float = 0

    def data(self) -> Dict[str, Any]:
        self.full_percent = round(self.full / self.all * 100, 2)
        self.partial_percent = 100 - self.full_percent - round(self.empty / self.all * 100, 2)
        self.empty_percent = 100 - self.full_percent - self.partial_percent

        stat_min_percent = 5
        self.stat_full_percent = (
            100
            - (max(self.empty_percent, stat_min_percent) if self.empty_percent else 0)
            - (max(self.partial_percent, stat_min_percent) if self.partial_percent else 0)
        )
        self.stat_partial_percent = (
            100
            - self.stat_full_percent
            - (max(self.empty_percent, stat_min_percent) if self.empty_percent else 0)
        )
        self.stat_empty_percent = 100 - self.stat_full_percent - self.stat_partial_percent

        return vars(self)

    def increase_all(self) -> None:
        self.all += 1

    def increase_full(self, method: SpecMethod) -> None:
        self.full += 1
        self.methods_full.append(DiffMethod(method))

    def increase_partial(self, method: SpecMethod, details: List[str]) -> None:
        self.partial += 1
        self.methods_partial.append(DiffMethod(method, details))

    def increase_empty(self, method: SpecMethod) -> None:
        self.empty += 1
        self.methods_empty.append(DiffMethod(method))


class Differ:
    def __init__(self, golden_spec: Dict[str, SpecMethod], testing_spec: Dict[str, SpecMethod]) -> None:
        self.golden_spec = golden_spec
        self.testing_spec = testing_spec
        self.diff = Diff()

    def get_diff(self) -> Diff:
        for method_id in self.golden_spec.keys():
            self.diff.increase_all()

            if self.is_method_not_covered(method_id):
                continue

            if self.is_partial_method(method_id):
                continue

            self.diff.increase_full(self.golden_spec[method_id])

        return self.diff

    def is_method_not_covered(self, method_id: str) -> bool:
        if method_id not in self.testing_spec:
            self.diff.increase_empty(self.golden_spec[method_id])
            return True
        return False

    def is_partial_method(self, method_id: str) -> bool:
        details = []

        diff_codes = set(self.golden_spec[method_id].response_codes) - set(self.testing_spec[method_id].response_codes)
        if diff_codes:
            details.append(f"Not covered status codes: {','.join(diff_codes)}")

        diff_queries = set(self.golden_spec[method_id].query_params) - set(self.testing_spec[method_id].query_params)
        if diff_queries:
            details.append(f"Not covered params: {','.join(diff_queries)}")

        if details:
            self.diff.increase_partial(self.golden_spec[method_id], details)
            return True
        return False



import os
from typing import Any

from jinja2 import Environment, FileSystemLoader, Template

from .differ import Diff


class Generator:
    _PATH_TEMPLATES = os.path.dirname(os.path.realpath(__file__)) + '/templates'
    _TEMPLATE_COVERAGE = 'coverage.html.j2'

    def __init__(self) -> None:
        self._templates = Environment(loader=FileSystemLoader(self._PATH_TEMPLATES))

    def _get_template(self, template_name: str) -> Template:
        return self._templates.get_template(name=template_name)

    @staticmethod
    def _create_dir(dirname: str) -> None:
        if dirname and not os.path.exists(dirname):
            os.makedirs(dirname)

    def _generate_by_template(self, file_path: str, template_name: str, **kwargs: Any) -> None:
        template = self._get_template(template_name=template_name)

        self._create_dir(os.path.dirname(file_path))
        with open(file_path, 'w') as file:
            file.write(template.render(**kwargs))

    def coverage_report(self, diff: Diff, file_path: str) -> None:
        self._generate_by_template(
            file_path=file_path,
            template_name=self._TEMPLATE_COVERAGE,
            **diff.data()
        )



from setuptools import setup


def find_required():
    with open("requirements.txt") as f:
        return f.read().splitlines()


setup(
    name="vedro-spec-compare",
    version="0.0.2",
    description="OpenAPI spec compare",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Konstantin Shefer",
    author_email="kostya.shefer.999@gmail.com",
    python_requires=">=3.8",
    url="https://github.com/kvs8/vedro-spec-compare",
    license="Apache-2.0",
    packages=['vedro_spec_compare'],
    install_requires=find_required(),
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    entry_points={
        "console_scripts": [
            "vsc = vedro_spec_compare:command",
        ],
    },
    include_package_data=True,
)



import argparse

from .coverage import coverage


def command() -> None:
    parser = argparse.ArgumentParser(description='vedro-spec-compare commands')
    subparsers = parser.add_subparsers(help='Available commands', required=True)

    coverage_parser = subparsers.add_parser('coverage', help='Generate coverage report')
    coverage_parser.add_argument('golden_spec_path', type=str, help='Path to the golden OpenAPI spec')
    coverage_parser.add_argument('testing_spec_path', type=str, help='Path to the testing OpenAPI spec')
    coverage_parser.add_argument(
        '--report-path', type=str, help='The path of the coverage report', default='coverage.html'
    )
    coverage_parser.set_defaults(func=coverage)

    args = parser.parse_args()
    args.func(args)



import logging
from typing import Any

from .differ import Differ
from .generator import Generator
from .parser import Parser


def coverage(args: Any) -> None:
    logging.basicConfig(level=logging.INFO, format='%(message)s')
    logger = logging.getLogger("Coverage")

    try:
        logger.info("Determination of the test coverage")

        logger.info(f"Parsing the golden spec: {args.golden_spec_path}")
        golden_spec_method = Parser.parse(args.golden_spec_path)

        logger.info(f"Parsing the testing spec: {args.testing_spec_path}")
        testing_spec_method = Parser.parse(args.testing_spec_path)

        logger.info("Defining the difference")
        diff = Differ(golden_spec_method, testing_spec_method).get_diff()

        logger.info(f"Generating the coverage report: {args.report_path}")
        Generator().coverage_report(diff, args.report_path)

    except FileNotFoundError as e:
        logger.critical(f"{e}")
